@@ -8,12 +8,12 @@
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
 # of the License, or (at your option) any later version.
-# 
+#
 # Linaro Image Tools is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with Linaro Image Tools; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
@@ -779,7 +779,8 @@ class FetchedPackageTests(TestCaseWithFixtures):
         for relationship, value in relationships.items():
             dummy_relationships[relationship.lower().replace('-', '_')] = value
         target_package = DummyFetchedPackage(
-            "foo", "1.0", content=open(deb_file_path).read(), **dummy_relationships)
+            "foo", "1.0", content=open(deb_file_path).read(),
+            **dummy_relationships)
         created_package = FetchedPackage.from_deb(deb_file_path)
         self.assertThat(created_package, MatchesPackage(target_package))
 
@@ -931,7 +932,6 @@ class AptCacheTests(TestCaseWithFixtures):
             open(os.path.join(
                 cache.tempdir, "var", "lib", "dpkg", "status")).read())
 
-
     def test_package_list_has_no_passwords(self):
         source1 = self.useFixture(AptSourceFixture([]))
         path = re.sub("file:/", "file://user:pass@", source1.sources_entry)
@@ -947,6 +947,7 @@ class AptCacheTests(TestCaseWithFixtures):
         sources_list = open(sources_list_location).read()
         self.assertEqual("deb %s\n" % source1.sources_entry, sources_list)
 
+
 class PackageFetcherTests(TestCaseWithFixtures):
 
     def test_context_manager(self):
