@@ -7,6 +7,7 @@ from linaro_image_tools.media_create.tests import (
     )
 from linaro_image_tools.utils import has_command
 
+
 def test_suite():
     module_names = [
         'linaro_image_tools.tests.test_cmd_runner',
@@ -14,11 +15,14 @@ def test_suite():
         ]
     # if pyflakes is installed and we're running from a bzr checkout...
     if has_command('pyflakes') and not os.path.isabs(__file__):
-        # ...also run the pyflakes tests
+        # ...also run the pyflakes test
         module_names.append('linaro_image_tools.tests.test_pyflakes')
+    # if pep8 is installed and we're running from a bzr checkout...
+    if has_command('pep8') and not os.path.isabs(__file__):
+        # ...also run the pep8 test
+        module_names.append('linaro_image_tools.tests.test_pep8')
     loader = unittest.TestLoader()
     suite = loader.loadTestsFromNames(module_names)
     suite.addTests(hwpack_suite())
     suite.addTests(media_create_suite())
     return suite
-
